/*  SQuaLe
 *
 *  Copyright (C) 2004 Julien Moutte <julien@moutte.net>
 *
 *  squaleoracleworker.c : Source for SqualeOracleWorker object.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <sqlora.h>
#include <glib-object.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_DMALLOC
#include <dmalloc.h>
#endif

#define ORA_LOGIN "user"
#define ORA_PASSWD "pass"
#define ORA_TNS "TNS"
#define ORA_QUERY "SELECT 1 FROM DUAL"

int main (int argc, char **argv)
{
  char *conn_string = NULL;
  sqlo_db_handle_t dbh;
  sqlo_stmt_handle_t sth = SQLO_STH_INIT;
  const char **col_names = NULL;
  
  if (SQLO_SUCCESS != sqlo_init (SQLO_ON, 32767, 1)) {
    g_warning ("Failed initing libsqlora8");
    exit (0);
  }
  
  conn_string = g_strdup_printf ("%s/%s@%s", ORA_LOGIN, ORA_PASSWD, ORA_TNS);
  
  if (SQLO_SUCCESS != sqlo_connect (&dbh, conn_string)) {
    g_warning ("Connection attempt to Oracle database %s failed: %s",
               conn_string, sqlo_geterror (dbh));
    exit (0);
  }
  else {
    g_message ("Successfully connected to Oracle");
  }
  
  if (sqlo_open2 (&sth, dbh, ORA_QUERY, 0, NULL) < 0) {
    g_warning ("SQL query %s failed : %s", ORA_QUERY, sqlo_geterror (dbh));
  }
  else {
    gint num_fields, status, i;
    
    g_message ("SQL query succeeded");
    
    col_names = sqlo_ocol_names (sth, &num_fields);
    
    for (i = 0; i < num_fields; i++) {
      g_print ("|\t%s\t|", col_names[i]);
    }
    g_print ("\n");
    while (SQLO_SUCCESS == (status = (sqlo_fetch (sth, 1)))) {
      const char **v = sqlo_values (sth, NULL, 1);
      for (i = 0; i < num_fields; i++) {
        g_print ("|\t%s\t|", v[i]);
      }
      g_print ("\n");
    }
    
    g_print ("status %d and SQLO_SUCCESS %d SQLO_NO_DATA %d SQLO_STILL_EXECUTING %d\n", status, SQLO_SUCCESS, SQLO_NO_DATA, SQLO_STILL_EXECUTING);
    
    if (status != 0 ) {
      g_warning ("An error occured while fetching: %s", sqlo_geterror (dbh));
    }
    else if (status == SQLO_NO_DATA) {
      g_warning ("No data");
    }
    
    if (sqlo_close (sth) != SQLO_SUCCESS) {
      g_warning ("Failed closing cursor: %s", sqlo_geterror (dbh));
    }
  }
  
  g_message ("Disconnecting from Oracle");
  sqlo_finish (dbh);
  
  exit (0);
}
